import os
import sys

modelFolder = os.path.join(os.path.dirname(__file__), 'trained_model/')
sys.path.append(os.path.join(os.path.dirname(__file__), 'model'))
sys.path.append(os.path.join(os.path.dirname(__file__), 'utils'))

import cv2
import numpy as np
import torch
from torch.autograd import Variable
from torchvision.utils import make_grid
from utils_SH import get_shading

# ---------------- create normal for rendering half sphere ------
img_size = 256
x = np.linspace(-1, 1, img_size)
z = np.linspace(1, -1, img_size)
x, z = np.meshgrid(x, z)

mag = np.sqrt(x**2 + z**2)
valid = mag <= 1
y = -np.sqrt(1 - (x * valid)**2 - (z * valid)**2)
x = x * valid
y = y * valid
z = z * valid
normal = np.concatenate((x[..., None], y[..., None], z[..., None]), axis=2)
normal = np.reshape(normal, (-1, 3))
# -----------------------------------------------------------------

modelFolder = os.path.join(os.path.dirname(__file__), 'trained_model/')
lightFolder = os.path.join(os.path.dirname(__file__), 'data', 'example_light')


def get_model(sz=512):
    if sz == 512:
        from defineHourglass_512_gray_skip import HourglassNet
        net = HourglassNet()
        net.load_state_dict(
            torch.load(os.path.join(modelFolder, 'trained_model_03.t7')))
    elif sz == 1024:
        from defineHourglass_1024_gray_skip_matchFeature import (
            HourglassNet, HourglassNet_1024)
        net = HourglassNet_1024(HourglassNet(16), 16)
        net.load_state_dict(
            torch.load(os.path.join(modelFolder, 'trained_model_1024_03.t7')))
    else:
        raise ValueError(sz)
    # net.cuda()
    net.train(False)
    return net


def get_resize_img(path, size):
    img = cv2.imread(path)
    Y, X, _ = img.shape
    img = cv2.resize(img, (size, size))
    img = cv2.cvtColor(img, cv2.COLOR_BGR2LAB)
    return img, Y, X


def get_light(sh_raw):
    sh_raw = sh_raw * .7
    shading = get_shading(normal, sh_raw)
    value = np.percentile(shading, 95)
    ind = shading > value
    shading[ind] = value
    shading = (shading - np.min(shading)) / (np.max(shading) - np.min(shading))
    shading = (shading * 255.0).astype(np.uint8)
    shading = np.reshape(shading, (256, 256))
    shading = shading * valid
    return shading


def render(path, sh, size):
    img, Y, X = get_resize_img(path, size)
    net = get_model(size)
    sh_target = get_light(sh)
    inputL = (img[:, :, 0].astype(np.float32) / 255).transpose(0, 1)[None,
                                                                     None]
    inputL = Variable(torch.from_numpy(inputL))
    sh = np.reshape(sh, (1, 9, 1, 1)).astype(np.float32)
    sh = Variable(torch.from_numpy(sh))
    outL, sh_est_raw = net(inputL, sh, 0)
    outL = np.round(outL[0].detach().data.numpy().transpose(1, 2, 0) *
                    255).astype(np.uint8)
    outL = np.squeeze(outL)
    out = img.copy()
    out[:, :, 0] = outL
    out = cv2.cvtColor(out, cv2.COLOR_LAB2BGR)
    out = cv2.resize(out, (X, Y))
    sh_est_raw = sh_est_raw.detach().numpy().squeeze()
    sh_est = get_light(sh_est_raw)
    return out, sh_target, sh_est
